from krita import *
from PyQt5.QtWidgets import *
import json
from pathlib import Path
from threading import Thread

class AutoExport(DockWidget):
    def __init__(self):
        super().__init__()
        settings = dict(alpha=True,
                        compression=5,
                        forceSRGB=False,
                        indexed=True,
                        interlaced=True,
                        saveSRGBProfile=False,
                        transparencyFillcolor=[0, 0, 0])
        self.infoObject = InfoObject()
        for k, v in settings.items():
            self.infoObject.setProperty(k, v)
        self.setWindowTitle("Auto Export")
        self.activeNode = None
        self.flushed = False
        self.selectedLayers = {}
        mainWidget = QWidget(self)
        textInput = QTextEdit()
        self.textInput = textInput
        textInput.textChanged.connect(self.on_change)
        self.label = QLabel()
        self.checkbox = QCheckBox("Export on save")
        self.checkbox.setChecked(True)
        layout = QVBoxLayout(mainWidget)
        layout.addWidget(self.label)
        layout.addWidget(self.checkbox)
        layout.addWidget(textInput)
        self.setWidget(mainWidget)
        self.timer = QTimer()
        self.timer.timeout.connect(self.update)
        self.timer.start(33)
        self.saveTimer = QTimer()
        self.saveTimer.timeout.connect(self.connectToSaveEvent)
        self.saveTimer.start(33)
    def connectToSaveEvent(self):
        for a in Krita.instance().actions():
            if a.text() == "&Save":
                a.triggered.connect(self.export)
        self.saveTimer.stop()
    def readDocumentFile(self, doc):
        filePath = doc.fileName()
        if filePath:
            Path(filePath + ".autoexport").touch(exist_ok=True)
            with open(filePath + ".autoexport", "r") as f:
                saved = {}
                try:
                    saved = json.load(f)
                except json.JSONDecodeError:
                    pass
                self.selectedLayers = saved
    def update(self):
        doc = Krita.instance().activeDocument()
        if not doc:
            return
        if self.activeNode == doc.activeNode():
            return
        self.flushed = True
        self.readDocumentFile(doc)
        self.activeNode = doc.activeNode()
        n = self.activeNode.name()
        self.label.setText(n)
        if n in self.selectedLayers: 
            self.textInput.setText(self.selectedLayers[n])
        else:
            self.textInput.setText("")

    def on_change(self):
        if self.flushed:
            self.flushed = False
            return
        doc = Krita.instance().activeDocument()
        self.readDocumentFile(doc)
        n = doc.activeNode().name()
        self.selectedLayers[n] = self.textInput.toPlainText()
        filePath = doc.fileName()
        with open(filePath + ".autoexport", "w") as f:
            json.dump(self.selectedLayers, f)
    def canvasChanged(self, canvas):
        pass

    def export(self):
        if not self.checkbox.isChecked():
            return
        instance = Krita.instance()
        doc = instance.activeDocument()
        self.readDocumentFile(doc)
        export_layers = {}
        for k, v in self.selectedLayers.items():
            for f in v.split("\n"):
                if not f in export_layers:
                    export_layers[f] = [k]
                else:
                    export_layers[f].append(k)
        for f, v in export_layers.items():
            newDoc = instance.createDocument(doc.width(),
                                             doc.height(),
                                             f,
                                             doc.colorModel(),
                                             doc.colorDepth(),
                                             doc.colorProfile(),
                                             doc.resolution())
            root = newDoc.rootNode()
            topNodes = doc.topLevelNodes()
            for n in newDoc.topLevelNodes():
                n.remove()
            for n in topNodes:
                for nodeName in v:
                    if n.name() == nodeName:
                        root.addChildNode(n.clone(), None)
            newDoc.setBatchmode(True)
            directory, oldName = os.path.split(doc.fileName())
            newPath = os.path.join(directory, f + ".png")
            newDoc.exportImage(newPath, self.infoObject)
            newDoc.close()

Krita.instance().addDockWidgetFactory(DockWidgetFactory("Auto Export",
                                      DockWidgetFactoryBase.DockRight,
                                      AutoExport
                                      ))
